#!/bin/bash
#############################################
# USER INPUT

# Set the time interval for visbility computations
#DATE_START="2016-07-01 00:00"
#DATE_STOP="2017-06-30 23:59"

# Short time interval for tests
DATE_START="2016-12-01 00:00"
DATE_STOP="2016-12-01 00:59"

# Remember that Solar ephemeris in sun_from_horizons.dat are only until 2021-10-21
# The script will not be able to work after this date unless you update sun_from_horizons.dat

# Automatically get the latest orbit
RA_ORBIT_FILE="automatic"
# or you may specify the orbit file manually, for example:
#RA_ORBIT_FILE="/tmp/RA151115-180604.org"
### Number of fakerat threads running in parallell
#NUMBER_OF_THREADS=6
NUMBER_OF_THREADS=1
#############################################
### You probably do not want to change anything below this line
#############################################
###
FAKERAT_PATH=`readlink -f $0`
FAKERAT_PATH=`dirname $FAKERAT_PATH`
FAKERATLIB_PATH=`dirname $FAKERAT_PATH`/fakerat_lib_4.3.6_v1
#############################################

############### The following will test if the needed external programs are installed ###############
SCRIPTNAME=`basename $0`
MISSING_PROGRAM=0
LIST_OF_MISSING_PROGRAMS=""

#################### Check SCHED ####################
# Find the SCHED executable
SCHEDEXECUTABLE=`which sched 2>/dev/null`
if [ -z $SCHEDEXECUTABLE ];then
 if [ -x $SCHED/bin/sched ];then
  SCHEDEXECUTABLE=$SCHED/bin/sched
 else
  # Yes it will not work, but the error message will look meaningful
  SCHEDEXECUTABLE="sched"
 fi
fi
# export it just in case... we are gonna use it from inside a forked function
export SCHEDEXECUTABLE
#####################################################

for TESTED_PROGRAM in gcc awk sed bc cat touch head tail grep skycoor readlink curlftpfs date $SCHEDEXECUTABLE ;do
 if ! which $TESTED_PROGRAM &>/dev/null ;then
  MISSING_PROGRAM=1
  LIST_OF_MISSING_PROGRAMS="$LIST_OF_MISSING_PROGRAMS  $TESTED_PROGRAM"
  echo -e "An external program $TESTED_PROGRAM - \033[01;31mNOT found\033[00m"
  if [ "$TESTED_PROGRAM" == "skycoor" ];then
   echo "skycoor program is part of WCSTools, see http://tdc-www.harvard.edu/wcstools/"
  fi
  if [ "$TESTED_PROGRAM" == "sched" ];then
   echo "NRAO SCHED may be found here: http://www.aoc.nrao.edu/software/sched/"
  fi
 fi
done
if [ $MISSING_PROGRAM -eq 1 ] ;then
 echo -e "
ERROR: some programs needed to run $SCRIPTNAME were not found in the \$PATH!
\033[01;31mPlease install the following programs:

$LIST_OF_MISSING_PROGRAMS\033[00m
"
 exit 1
fi

#####################################################################################################

## This function will compare two files by computing their MD5 checksums with md5sum
function check_if_two_files_are_the_same {
 FILE1="$1"
 FILE2="$2"
 which md5sum &>/dev/null
 if [ $? -ne 0 ];then
  echo "Cannot find the command: md5sum" >> /dev/stderr
  echo "Assuming files $FILE1 and $FILE2 are different" >> /dev/stderr
  return 0
 fi
 if [ ! -f "$FILE1" ];then
  return 0
 fi
 if [ ! -f "$FILE2" ];then
  return 0
 fi
 MD5SUM_OF_FILE1=`md5sum $FILE1 | awk '{print $1}'`
 MD5SUM_OF_FILE2=`md5sum $FILE2 | awk '{print $1}'`
 if [ "$MD5SUM_OF_FILE1" != "$MD5SUM_OF_FILE2" ];then
  echo "Files $FILE1 and $FILE2 are not identical" >> /dev/stderr
  return 0
 fi
 return 1
}

## This function will set the random session key in attempt to avoid 
## file collisions if other instances of the script are running at the same time.
function set_session_key {
 if [ -r /dev/urandom ];then
  local RANDOMFILE=/dev/urandom
 elif [ -r /dev/random ];then
  local RANDOMFILE=/dev/random
 else
  echo "ERROR: cannot find /dev/random" >> /dev/stderr
  local RANDOMFILE=""
 fi
 if [ "$RANDOMFILE" != "" ];then
  local SESSION_KEY="$$"_`tr -cd a-zA-Z0-9 < $RANDOMFILE | head -c 12`
 else
  local SESSION_KEY="$$"
 fi
 echo "$SESSION_KEY"
}



## This function will get the latest long-term orbit prediction to compute distance from a tracing station
function get_latest_orbit {
 local FTPURL_ORBIT_DIR_ENDS_WITH_SLASH='ftp://radik:Zap069_qq@webinet.asc.rssi.ru/radioastron/oddata/predict/long/'
 # The global variable SESSION_KEY should be set externally
 if [ -z $SESSION_KEY ];then
  echo "ERROR: SESSION_KEY variable is not set!" >> /dev/stderr
  exit 1
 fi
 local CURLFTPFS_MOUNT_DIR_ORBIT="/tmp/autofake_sh_get_latest_orbit_$SESSION_KEY"
 # Create the directory if it does not exist
 if [ ! -d $CURLFTPFS_MOUNT_DIR_ORBIT/ ];then
  mkdir $CURLFTPFS_MOUNT_DIR_ORBIT/
 else
  fusermount -u $CURLFTPFS_MOUNT_DIR_ORBIT/
 fi
 # mount read-only
 curlftpfs -r "$FTPURL_ORBIT_DIR_ENDS_WITH_SLASH" $CURLFTPFS_MOUNT_DIR_ORBIT/
 if [ $? -ne 0 ];then
  echo "WARNING: cannot mount $FTPURL_ORBIT_DIR_ENDS_WITH_SLASH" >> /dev/stderr
 else
  # Check if there are orbit files in the remote directory
  for REMOTEORBITFILE in $CURLFTPFS_MOUNT_DIR_ORBIT/RA*.org ;do
   if [ -f $REMOTEORBITFILE ];then
    echo "OK, found some orbit files on the FTP server" >> /dev/stderr
    local ORBITFILE=`ls -t $CURLFTPFS_MOUNT_DIR_ORBIT/RA*.org | head -n1`
    local BASENAME_ORBITFILE=`basename $ORBITFILE`
    echo "The latest orbit file found on the FTP server is $BASENAME_ORBITFILE" >> /dev/stderr
    echo "We are in $PWD, checking if there are local orbit files..." >> /dev/stderr
    # Check if there are local orbit files
    for LOCALORBITFILE in RA*.org ;do
     if [ -f $LOCALORBITFILE ];then
      local LATESTLOCALORBITFILE=`ls -t RA*.org | head -n1`
      echo "The latest local orbit file is $LATESTLOCALORBITFILE" >> /dev/stderr
      # If there is a newer orbit file
      if [ "$LATESTLOCALORBITFILE" != "$BASENAME_ORBITFILE" ];then
       echo "The latest local file is not the same as the latest remote one: $LATESTLOCALORBITFILE (local),  $BASENAME_ORBITFILE (remote)" >> /dev/stderr
       # Copy the remore file cutting-off its header
       echo -e "\033[01;34mDownloading the new orbit file $BASENAME_ORBITFILE\033[00m
It \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
       cp -v $ORBITFILE new_orbit$SESSION_KEY.org >> /dev/stderr
       if [ $? -eq 0 ];then
        # Check if the copy is identical to the original by computing checksums
        # Yes, sadly, for this we have to download the original again
        echo -e "Checking if the orbit file is downloaded correctly... 
This check \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
        check_if_two_files_are_the_same $ORBITFILE new_orbit$SESSION_KEY.org
        if [ $? -eq 1 ];then 
         echo -e "\033[01;32mYes, the orbit file is downloaded correctly.\033[00m" >> /dev/stderr
         cat new_orbit$SESSION_KEY.org | tail -n +15 > "$BASENAME_ORBITFILE"
         # OK, now we have the newest orbit file locally
        else
         echo -e "\033[01;31mNO\033[00m, the downlaoded file did not pass the check! The corrupted download will be removed." >> /dev/stderr
        fi
       else
        echo "ERROR copying the new orbit file!" >> /dev/stderr
       fi # if cp -v $ORBITFILE new_orbit$SESSION_KEY.org is OK
       # Anyway, remove the downloaded file
       rm -f new_orbit$SESSION_KEY.org
       # end of remote file copy
      fi
     else
      # No local orbit files found!

      # Copy the remore file cutting-off its header
      echo -e "\033[01;34mDownloading the new orbit file $BASENAME_ORBITFILE\033[00m
It \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
      cp -v $ORBITFILE new_orbit$SESSION_KEY.org >> /dev/stderr
      if [ $? -eq 0 ];then
       # Check if the copy is identical to the original by computing checksums
       # Yes, sadly, for this we have to download the original again
       echo -e "Checking if the orbit file is downloaded correctly... 
This check \033[01;34mwill take time\033[00m as the orbit file is large. 
Please, \033[01;34mDO NOT INTERRUPT THIS PROCESS!\033[00m" >> /dev/stderr
       check_if_two_files_are_the_same $ORBITFILE new_orbit$SESSION_KEY.org
       if [ $? -eq 1 ];then 
        echo -e "\033[01;32mYes, the orbit file is downloaded correctly.\033[00m" >> /dev/stderr
        cat new_orbit$SESSION_KEY.org | tail -n +15 > "$BASENAME_ORBITFILE"
        # OK, now we have the newest orbit file locally
       else
        echo -e "\033[01;31mNO\033[00m, the downlaoded file did not pass the check! The corrupted download will be removed." >> /dev/stderr
       fi
      else
       echo "ERROR copying the new orbit file!" >> /dev/stderr
      fi # if cp -v $ORBITFILE new_orbit$SESSION_KEY.org is OK
      # Anyway, remove the downloaded file
      rm -f new_orbit$SESSION_KEY.org
      # end of remote file copy
     fi
     break
    done # for LOCALORBITFILE in RA*.org ;do
   else
    echo "ERROR: no orbit files RA*.org could be found on the FTP server" >> /dev/stderr
    # Will try to use the local ones
   fi # if [ -f $REMOTEORBITFILE ];then
   break # anyway
  done # for REMOTEORBITFILE in $CURLFTPFS_MOUNT_DIR_ORBIT/RA*.org ;do
   
 fi # if curlftpfs mounted well

 ### Find the latest orbit prediction file, we may have just updated it
 # Check if there are local orbit files
 for LOCALORBITFILE in RA*.org ;do
  if [ -f $LOCALORBITFILE ];then
   echo "Found orbit files locally" >> /dev/stderr
   local ORBITFILE=`ls -t RA*.org | head -n1`
   echo "The latest orbit file is $ORBITFILE" >> /dev/stderr
   break
  else
   echo "ERROR: no orbit files RA*.org could be found" >> /dev/stderr
   exit 1
  fi
 done

 ### Clean up 
 # Just for the paranoia sake, because we remove the directory recursively below
 if [ "$CURLFTPFS_MOUNT_DIR_ORBIT" == "" ];then
  echo "FATAL ERROR: corrupted variable CURLFTPFS_MOUNT_DIR_ORBIT" >> /dev/stderr
  exit 1
 fi
 if [ "$CURLFTPFS_MOUNT_DIR_ORBIT" == "/" ];then
  echo "FATAL ERROR: corrupted variable CURLFTPFS_MOUNT_DIR_ORBIT" >> /dev/stderr
  exit 1
 fi
 fusermount -u $CURLFTPFS_MOUNT_DIR_ORBIT/ && rm -rf $CURLFTPFS_MOUNT_DIR_ORBIT

 # Return the result
 echo "$ORBITFILE"
}


############### This pice is to compute visibility for ALL sources ###############
if [ -z $1 ];then
 # If no parameters are specified - compute visibility for all sources in autosched.sh
 echo "Removing old computation results" >> /dev/stderr
 rm -f results.txt processing_complete.txt
 #rm -f source_visibility_prediction__*.txt
 PROCESSING_START=`date +%s`
 # Re-compile C programs
 gcc -O3 -o check_gb_ha check_gb_ha.c -lm
 gcc -O3 -o check_pu_altaz check_pu_altaz.c
 gcc -O3 -o fort_75_to_baseline_and_pa fort_75_to_baseline_and_pa.c -lm
 # Set session key
 export SESSION_KEY=$(set_session_key) # or SESSION_KEY=`set_session_key`
 ### Get the latest orbit ###
 STARTDIR=$PWD
 if [ ! -d orbit ];then
  mkdir orbit
 fi
 cd orbit
 if [ "$RA_ORBIT_FILE" = "automatic" ];then
  # automatically get the latest orbit file
  RA_ORBIT_FILE=$(get_latest_orbit)
  # Delete the oldest orbit file
  OLDEST_ORBIT_FILE=`ls -t RA*.org | tail -n 1`
  # make sure the oldest orbit file is not the same as the new one
  if [ "$OLDEST_ORBIT_FILE" != "$RA_ORBIT_FILE" ];then
   rm -f "$OLDEST_ORBIT_FILE"
  else
   echo "Not deleting the old orbit file - it's the same as the new one!" >> /dev/stderr
  fi
 fi
 # 
 # Check if the orbit file is here
 if [ ! -s "$RA_ORBIT_FILE" ];then
  echo "ERROR: cannot find the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
  exit 1
 else
  echo "Using the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
 fi
 # Check if we need to chop-off it's head
 head -n 10 $RA_ORBIT_FILE |grep "META_START" &> /dev/null
 if [ $? -eq 0 ];then
  LOCALCOPY_ORBITFILE_NOHEADER=`basename $RA_ORBIT_FILE .org`_noheader.org
  echo "WARNING: the predicted orbit file $RA_ORBIT_FILE contains a header, will make a loacal copy without a header called $LOCALCOPY_ORBITFILE_NOHEADER" >> /dev/stderr
  tail -n +15 $RA_ORBIT_FILE > "$LOCALCOPY_ORBITFILE_NOHEADER"
  RA_ORBIT_FILE="$LOCALCOPY_ORBITFILE_NOHEADER"
 fi
 rm -f ra_orbit
 ln -s $RA_ORBIT_FILE ra_orbit
 # double-check if the orbit file is here
 if [ ! -s "$RA_ORBIT_FILE" ];then
  echo "ERROR: cannot find the predicted orbit file $RA_ORBIT_FILE" >> /dev/stderr
  exit 1
 fi
 cd $STARTDIR
 ### Done with the orbit thing ###
 # Cycle through all sources listed inside autosched.sh
 COUNTER=0
 NPROC=0
 mkdir /tmp/autofake_"$SESSION_KEY"_results
 grep "source=" autosched.sh | grep -v "!    source=" | grep -v "!     source=" | grep -v "!   source=" | grep -v "!  source=" | grep -v "! source=" | grep -v "!source=" | grep -v 'calibrator' | grep -v "fake source representing RadioAstron" | awk '{print $1}' | while read TARGET ;do
  # Each time we are about to start a new thread, touch all the results files
  # to prevent evil tmpwatch from removing the older results
  touch /tmp/autofake_"$SESSION_KEY"_results
  for i in /tmp/autofake_"$SESSION_KEY"_results/* ;do
   if [ -f $i ];then
    touch $i
   fi
  done
  #
  CURRENT_RESULTS_FILE=/tmp/autofake_"$SESSION_KEY"_results/results_`echo "$COUNTER" | awk '{printf "%05d",$1}'`.txt
  COUNTER=$[$COUNTER+1]
  ############################
  TARGET=${TARGET/"source="/}
  TARGET=${TARGET//"'"/}
  TARGET=${TARGET//","/}
  TARGET=`echo "$TARGET" | awk '{print $1}'`
  # Go multithread
  $0 $TARGET > $CURRENT_RESULTS_FILE &
  PIDS="$PIDS $! "
  NPROC=$[$NPROC+1]
  while [ $NPROC -gt $[$NUMBER_OF_THREADS-1] ] ;do
   # check each running process
   #echo "$PIDS"
   for PID in $PIDS ;do
    ps -p $PID &> /dev/null
    if [ $? -ne 0 ];then
     PIDS=${PIDS/" $PID "/}
     NPROC=$[$NPROC-1]
    fi
   done
   sleep 0.01
  done
  # Now we are sure no more than NUMBER_OF_THREADS procssess are runnning
  echo "$PIDS" > /tmp/autofake_"$SESSION_KEY"_pids.tmp
 done
 
 # Wait for the remeining processes to finish
 for PID in `cat /tmp/autofake_"$SESSION_KEY"_pids.tmp` ;do
  while true ;do
   ps -p $PID &> /dev/null
   if [ $? -ne 0 ];then
    break
   fi
   sleep 0.01
  done
 done
 rm -f /tmp/autofake_"$SESSION_KEY"_pids.tmp

 # Combine individual source results into a single output file 
 for i in /tmp/autofake_"$SESSION_KEY"_results/results_*.txt ;do
  cat $i
 done > results.txt
 echo "Results are written to results.txt" >> /dev/stderr
 ## Generate a good output filename
 OUTFILENAME_ORBIT_START_DATE=`ls -l orbit/ra_orbit | awk '{print $2}' FS='->' | awk '{print $1}' | awk '{print $1}' FS='-'`
 OUTFILENAME_DATES="$DATE_START $DATE_STOP"
 OUTFILENAME_DATES="${OUTFILENAME_DATES// /_}"
 OUTFILENAME_DATES="${OUTFILENAME_DATES//:/}"
 OUTFILENAME="source_visibility_prediction__"$OUTFILENAME_DATES"__"$OUTFILENAME_ORBIT_START_DATE".txt"
 cp results.txt $OUTFILENAME
 echo "Results are copied from results.txt to $OUTFILENAME" >> /dev/stderr
 ##
 if [ -d autofake_results_dir ];then
  rm -rf autofake_results_dir &>/dev/null
 fi
 cp -r /tmp/autofake_"$SESSION_KEY"_results/  autofake_results_dir
 echo "Details are saved to autofake_results_dir" >> /dev/stderr
 date > processing_complete.txt
 PROCESSING_STOP=`date +%s`
 COMPUTING_TIME=`echo "($PROCESSING_STOP-$PROCESSING_START)/86400" | bc -ql | awk '{printf "%.3f",$1}'`
 echo "Processing time: $COMPUTING_TIME days"
 # Clean-up
 rm -rf /tmp/autofake_"$SESSION_KEY"_results/
 #
 exit 0
fi

############### What follows is needed to compute visibility for a given source ###############


############ Change to a temporary directory ############
WORK_DIR=/tmp/autofake_"$$"_`tr -cd a-zA-Z0-9 < /dev/urandom | head -c 12 ; echo ""`
mkdir $WORK_DIR
cd $WORK_DIR
# symlink all necessary files
for i in ground  menu_colors  menu_defaults  menu_defaults.1  orbit  sources.cat  stations.cat autosched.sh stations.dat sun_from_horizons.dat check_gb_ha check_pu_altaz fort_75_to_baseline_and_pa ;do
 ln -s "$FAKERAT_PATH"/"$i"
done
############

# playing with file descriptors to speed up while loop
exec 3<&0

SOURCE=$1
#
#############################################
# DERIVED PARAMETERS
DOY_START=`date --utc -d "$DATE_START" +%j`
UNIXSEC_START=`date --utc -d "$DATE_START" +%s`
DOY_STOP=`date --utc -d "$DATE_STOP" +%j`
UNIXSEC_STOP=`date --utc -d "$DATE_STOP" +%s`
#############################################

# get source position and name
RA=`grep "$SOURCE" autosched.sh | grep "source=" | head -n1 | awk '{print $2}' FS='RA=' | awk '{print $1}'`
DEC=`grep "$SOURCE" autosched.sh | grep "source=" | head -n1 | awk '{print $2}' FS='DEC=' | awk '{print $1}'`
SOURCEPOSITION_RADEC="$RA $DEC"
echo "Source position from autosched.sh: $SOURCEPOSITION_RADEC" >> /dev/stderr
SOURCENAME=$SOURCE

## Save source declination in degrees for future tests
SOURCE_DECLINATION_IN_DEG=`skycoor -d -j $SOURCEPOSITION_RADEC J2000 | awk '{print $2}'`

## Convert J2000 to B1950
SOURCEPOSITION_RADEC_B1950=`skycoor -b $SOURCEPOSITION_RADEC J2000`
echo "Source position converted to B1950: $SOURCEPOSITION_RADEC_B1950" >> /dev/stderr
SOURCEPOSITION_RA=`echo $SOURCEPOSITION_RADEC_B1950 | awk '{print $1}'`
SOURCEPOSITION_DEC=`echo $SOURCEPOSITION_RADEC_B1950 | awk '{print $2}'`

UNIXSEC=`echo "$UNIXSEC_START-3600"|bc -q`

# Time loop (the main loop)
while [ $UNIXSEC -lt $UNIXSEC_STOP ];do

 #echo -n "Investigating $SOURCENAME  " >> /dev/stderr

 #UNIXSEC=`echo "$UNIXSEC+3600"|bc -q`
 # Replacing with native BASH hoping that it might be faster
 UNIXSEC=$[$UNIXSEC+3600]

 DATE_FROM_UNIXSEC_FOR_SUN=`date --utc -d "@$UNIXSEC" +"%Y-%m-%d %H:%M"`
 SUN_RADEC=`grep "$DATE_FROM_UNIXSEC_FOR_SUN" sun_from_horizons.dat | awk '{print $3":"$4":"$5" "$6":"$7":"$8}'`
 SUN_ANGLE_ARCSEC=`skycoor -r $SUN_RADEC $SOURCEPOSITION_RADEC`
 # TEST IF THE SOLAR ANGLE IS PROMISING
 #echo "$DATE_FROM_UNIXSEC_FOR_SUN $SUN_RADEC $SOURCEPOSITION_RADEC $SUN_ANGLE_DEG" >> /dev/stderr
 #TEST=`echo "$SUN_ANGLE_DEG>92.0"|bc -ql`
 #if [ $TEST -eq 0 ];then
 # continue
 #fi
 # it was 160
 #TEST=`echo "$SUN_ANGLE_DEG<150.0"|bc -ql`
 #if [ $TEST -eq 0 ];then
 # continue
 #fi
 
 # Optimized version
 if [ `echo "$SUN_ANGLE_ARCSEC>80.5*3600.0 && $SUN_ANGLE_ARCSEC<168.0*3600.0"|bc -ql` -eq 0 ];then
  # If the angle is VERY bad, skip 10 days
  UNIXSEC=$[$UNIXSEC+864000]
  continue
 fi
 if [ `echo "$SUN_ANGLE_ARCSEC>89.5*3600.0 && $SUN_ANGLE_ARCSEC<159.0*3600.0"|bc -ql` -eq 0 ];then
  # If the angle is bad, skip one day
  UNIXSEC=$[$UNIXSEC+86400]
  continue
 fi
 if [ `echo "$SUN_ANGLE_ARCSEC>90.5*3600.0 && $SUN_ANGLE_ARCSEC<158.0*3600.0"|bc -ql` -eq 0 ];then
  # If the angle is just ouside limits, don't skip, try next time interval
  continue
 fi
 # Conduct the following computation only if the solar angle is good
 SUN_ANGLE_DEG=`echo "$SUN_ANGLE_ARCSEC/3600" | bc -ql | awk '{printf "%5.1f",$1}'` 

 # move it here so we compute this only if SOLAR ANGLE is good
 YEAR=`date --utc -d "@$UNIXSEC" +%Y`
 DOY=`date --utc -d "@$UNIXSEC" +%j`
 HOUR_START=`date --utc -d "@$UNIXSEC" +%H | awk '{printf "%02d",$1}'`
 UNIXSEC_HOUR_STOP=`echo "$UNIXSEC+3600"|bc -q`
 HOUR_STOP=`date --utc -d "@$UNIXSEC_HOUR_STOP" +%H |bc -q | awk '{printf "%02d",$1}'`

 # Check the sattelite visibility for the TS
 echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakesat <<EOD

NODATA
PLOTAZEL
TYPSAT = 2
TELEM      =
PUSCHINO
YEAR       = $YEAR
DAY        =  $DOY
START      = $HOUR_START:00:00
STOP       = $HOUR_STOP:00:00
FREQ       = 5000
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
INTEG      = 30
BNAME      = C
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
STATIONS   = RASTRON  , EFFELSBRG
LINEWIDTH  = 3
DOTSIZ     = 5
PLOT2F  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit
" > autofake-tmp-azel.csh
 nice csh autofake-tmp-azel.csh &> /dev/null
 
 #echo "################### DEBUG ###################" >>/dev/stderr
 #cat .fakesat_plotazel.dat autofake-tmp-azel.csh >>/dev/stderr
 #echo "#############################################" >>/dev/stderr
 
 rm -f azel_problem.tmp
 #grep "    1  " .fakesat_plotazel.dat | while read ONE AZ X EL Y RANGE REST ;do
 # A faster version 
 grep "    1  " .fakesat_plotazel.dat > /tmp/fast$$ 
 #cat /tmp/fast$$
 # playing with file descriptors to speed up while loop
 exec 0< /tmp/fast$$
 while read ONE AZ X EL Y RANGE REST ;do
  ./check_pu_altaz $EL $AZ
   if [ $? -eq 1 ];then
    touch azel_problem.tmp
    break
   fi
 #
 # if [ `echo "$AZ<350.0 && $AZ>10.0 && $EL>15.0 && $EL<80.0"|bc -ql` -eq 1 ];then
 #  continue
 # fi
 # touch azel_problem.tmp
 #
 done # parsing azel file
 # playing with file descriptors to speed up while loop
 exec 0<&3
 # remove tmp file
 rm -f /tmp/fast$$ &

 # Reset the variable
 ACTIVE_TS=""
 if [ ! -f azel_problem.tmp ];then
  ACTIVE_TS="PU"
 fi
# else
  # if Puschino TS doesn't see the satellite
  # test the GB TS

  rm -f azel_problem.tmp
  echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakesat <<EOD

NODATA
PLOTAZEL
TYPSAT = 2
TELEM      =
GBANK-5
YEAR       = $YEAR
DAY        =  $DOY
START      = $HOUR_START:00:00
STOP       = $HOUR_STOP:00:00
FREQ       = 5000
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
INTEG      = 30
BNAME      = C
RA         = 00:00:00 !dummy variable
DEC        = 00:00:00 !dummy variable
STATIONS   = RASTRON  , EFFELSBRG
LINEWIDTH  = 3
DOTSIZ     = 5
PLOT2F  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit
" > autofake-tmp-azel.csh
  nice csh autofake-tmp-azel.csh &> /dev/null
  
  rm -f azel_problem.tmp
  #grep "    1  " .fakesat_plotazel.dat | while read ONE AZ X EL Y RANGE REST ;do
  grep "    1  " .fakesat_plotazel.dat > /tmp/fast$$
  exec 0< /tmp/fast$$
  while read ONE AZ X EL Y RANGE REST ;do


  ./check_gb_ha $EL $AZ
  if [ $? -eq 1 ];then
   touch azel_problem.tmp
   break
  fi

  # The following test is implemented in C module
  #TEST=`echo "$EL<15"|bc -ql`
  #if [ $TEST -eq 1 ];then
  # touch azel_problem.tmp
  # break
  #fi
  
  done # parsing azel file

  # playing with file descriptors to speed up while loop
  exec 0<&3
  # remove tmp file
  rm -f /tmp/fast$$ 


  if [ ! -f azel_problem.tmp ];then
   ACTIVE_TS="$ACTIVE_TS GB"
#  else
#   ACTIVE_TS="NO"
#   continue
  fi
  
# fi # if PU doesn't see the satellite

 if [ "$ACTIVE_TS" == "" ];then
  continue
 fi

 # TS loop
 CONFIRMED_TS=""
 OK_TELESCOPE_SAVED=0
 OK_TELESCOPE=0
 #echo "ACTIVE_TS= $ACTIVE_TS" >> /dev/stderr
 for CURRENT_TS in $ACTIVE_TS ;do
  
 CURRENT_TS=${CURRENT_TS//PU/PUSCHINO}
 CURRENT_TS=${CURRENT_TS//GB/GBANK-5}
 
 if [ $OK_TELESCOPE -eq 1 ];then
  OK_TELESCOPE_SAVED=1
 fi
 OK_TELESCOPE=0
  

 # Telescope loop
 #for FAKERATTELESCOPE in EVPATORI EFFELSBR GBT ARECIBO USUDA ATCA ;do
 for FAKERATTELESCOPE in EFFELSBR GBT ATCA USUDA ;do

  #echo "LOOPING $ACTIVE_TS $FAKERATTELESCOPE $OK_TELESCOPE_SAVED"


  # Skip EFFELSBR if hopeless
  if [ "$FAKERATTELESCOPE" == "EFFELSBR" ];then
   TEST=`echo "$SOURCE_DECLINATION_IN_DEG<-32.0"|bc -ql`
   if [ $TEST -eq 1 ];then
    continue  
   fi
  fi

  # Skip ATCA if hopeless
  if [ "$FAKERATTELESCOPE" == "ATCA" ];then
   TEST=`echo "$SOURCE_DECLINATION_IN_DEG>47.0"|bc -ql`
   if [ $TEST -eq 1 ];then
    continue  
   fi
  fi

  # Skip GBT if hopeless
  if [ "$FAKERATTELESCOPE" == "GBT" ];then
   TEST=`echo "$SOURCE_DECLINATION_IN_DEG<-45.0"|bc -ql`
   if [ $TEST -eq 1 ];then
    continue  
   fi
  fi

  # Skip everythin else if hopeless OLD
  if [ "$FAKERATTELESCOPE" != "ATCA" ];then
   TEST=`echo "$SOURCE_DECLINATION_IN_DEG<-45.0"|bc -ql`
   if [ $TEST -eq 1 ];then
    continue  
   fi
  fi

  # Skip ARECIBO if hopeless
  if [ "$FAKERATTELESCOPE" == "ARECIBO" ];then
   TEST=`echo "$SOURCE_DECLINATION_IN_DEG>38.0"|bc -ql`
   if [ $TEST -eq 1 ];then
    continue  
   fi
   TEST=`echo "$SOURCE_DECLINATION_IN_DEG<0.0"|bc -ql`
   if [ $TEST -eq 1 ];then
    continue  
   fi
  fi

  # Remove temporary files from a previous run 
  rm -f .fakesat_plotazel.dat fort.75 fort.50

  # Launch the program
  echo "setenv PATH $FAKERAT_PATH:$FAKERATLIB_PATH:$PATH
fakesat <<EOD
UVPLOT
NODATA
TYPSAT = 2
CONSTRA
MODEL  model/none
SNRFLAG   -1.0
USEFWHM
LOGFWHM1     -1.30103004
LOGFWHM2       -1.30103004
LOGFWHM3    1.0
LOGS1        0.00000000
LOGS2        0.00000000
LOGS3    1.0
RATIO   1.0
PA        0.0
TELEM      =
$CURRENT_TS
YEAR       = $YEAR
DAY        =  $DOY
START      = $HOUR_START:00:00
STOP       = $HOUR_STOP:00:00
FREQ       = 5000
BAND      = 16
INTEG      = 1000
BNAME      = C
SOURCE     = TEST
RA         = $SOURCEPOSITION_RA
DEC        = $SOURCEPOSITION_DEC
STATIONS   = RASTRON  ,
$FAKERATTELESCOPE
LINEWIDTH  = 3
DOTSIZ     = 5
UMAX       =  5.00E9
VMAX       =  5.00E9
NGRDIL     = 5
NUP     = 2
PLOTF  /null
USEAEIOWM
AEIOWM(1)    171561556.00
AEIOWM(2) 0.9604
AEIOWM(3) 51.3815
AEIOWM(4) 331.000
AEIOWM(5) 292.000
AEIOWM(6) 0.010
YORB 2009
MORB  1
DORB   1
UTORB 0.00
/
EOD
exit

" > autofake-tmp.csh
  rm -f fort.75
  nice csh autofake-tmp.csh &>/dev/null
  # The following C insert will parse fort.75 and exit with 1 if something is wrong
  #echo "#################################################"
  #echo "$HOUR_START:00:00 $HOUR_STOP:00:00 $CURRENT_TS $FAKERATTELESCOPE"
  #echo "*************************************************"
  #cat fort.75
  #echo "#################################################"
  ./fort_75_to_baseline_and_pa &>/dev/null
  if [ $? -ne 0 ];then
   OK_TELESCOPE=0
   continue
  else
   OK_TELESCOPE=1
   CONFIRMED_TS="$CONFIRMED_TS $CURRENT_TS"
   BASE=`./fort_75_to_baseline_and_pa`
   #echo "CONFIRMED $OK_TELESCOPE"
   break
  fi 
  # Nothing to do here: continue or break above
done # Telescope loop

 if [ $OK_TELESCOPE -eq 1 ];then
  OK_TELESCOPE_SAVED=1
 fi

done # TS loop


# If not a single ground telescope was found
if [ $OK_TELESCOPE_SAVED -eq 0 ];then
 continue
fi

  CONFIRMED_TS=${CONFIRMED_TS/PUSCHINO/PU}
  CONFIRMED_TS=${CONFIRMED_TS/GBANK-5/GB}
  # Remove blank space
  CONFIRMED_TS=${CONFIRMED_TS//" "/}
  


  # Output the good visibility time
  DATE_FROM_UNIXSEC=`date --utc -d "@$UNIXSEC" +"%d.%m.%Y"`
  #echo -n "$UNIXSEC  $DATE_FROM_UNIXSEC $HOUR_START:00-$HOUR_STOP:00  $SOURCE  $SOURCENAME  $BASE"
  echo -n "$UNIXSEC  $DATE_FROM_UNIXSEC $HOUR_START:00-$HOUR_STOP:00  $SOURCENAME  $BASE"
  ALL_TELESCOPES=`nice ./autosched.sh $DATE_FROM_UNIXSEC $HOUR_START:00 $SOURCE 60`

 echo "  $SUN_ANGLE_DEG $CONFIRMED_TS  $ALL_TELESCOPES"
 
done # Time loop

cd - &> /dev/null
rm -rf $WORK_DIR
